﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using System.IO.Ports;
using System.Threading;
using System.Windows.Threading;


namespace ZigBeeCommunication
{
    /// <summary>
    /// Desktop client for zigbee communication
    /// </summary>
    public partial class MainWindow : Window
    {
        // Serial port to the zigbee module
        private SerialPort xbeeSerialport;

        // The COM port
        public static string COM_PORT = "COM5";


        /// <summary>
        /// Constructor
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();
            InitializeConnection();          
        }



        /// <summary>
        /// Initialize zigbee serial port
        /// </summary>
        private void InitializeConnection()
        {
            this.xbeeSerialport = new SerialPort(MainWindow.COM_PORT, 9600, Parity.None, 8, StopBits.One);
            this.xbeeSerialport.Handshake = Handshake.None;
            this.xbeeSerialport.DataReceived += new SerialDataReceivedEventHandler(xbeeSerialport_DataReceived);
            this.xbeeSerialport.Open();
        }


        /// <summary>
        /// Receiving data from the zigbee
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void xbeeSerialport_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            //Thread.Sleep(50);
            string data = this.xbeeSerialport.ReadLine();

            // Add incoming message to text box
            this.Dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(delegate() {
                this.incomingMessages.Text = data + Environment.NewLine + this.incomingMessages.Text;
            })); 

            //SendMessage(data);
            //this.distanceString = data;
            return;
        }


        /// <summary>
        /// Send a message to the remote zigbee
        /// </summary>
        /// <param name="message"></param>
        private void SendMessage(string message)
        {
            try
            {
                // Check if serial port is open; if not, then open serial port
                if (!this.xbeeSerialport.IsOpen)
                    this.xbeeSerialport.Open();

                //this.xbeeSerialport.DiscardOutBuffer();

                // send message
                byte[] messageInBytes = Encoding.UTF8.GetBytes(message + "\n");
                this.xbeeSerialport.Write(messageInBytes, 0, messageInBytes.Length);
                
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error opening/writing to serial port :: " + ex.Message, "Error!");
            }
        }


        /// <summary>
        /// Send "on" command to all zigbees
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStart_Click(object sender, RoutedEventArgs e)
        {
            this.SendMessage("on");
        }


        /// <summary>
        /// Send "off" command to all zigbees
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonStop_Click(object sender, RoutedEventArgs e)
        {
            this.SendMessage("off");
        }


        /// <summary>
        /// Exit application
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonExit_Click(object sender, RoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }


        /// <summary>
        /// Clear the textbox
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClear_Click(object sender, RoutedEventArgs e)
        {
            this.incomingMessages.Text = "";
        }
    }
}
